/* VirtuinoSE example: Client code to communicate with VirtuinoSE server code
 * Supported boards: * Supported boards: Arduino Uno or Nano + ESP8266 module 
 * Created by Ilias Lamprou
 * Updated Noe 1 2018
 * More examples: http://virtuino.com/index.php/virtuino-se/20-virtuino-se-code-examples
 */
//-----------------SETTINGS -------------------
String ssid     = "WIFI NETWORK";            
String password = "PASSWORD";

const char* virtuinoServerIP = "192.168.1.150";    // The server IP address
int virtuinoServerPort = 8000;                     // The server port number
String virtuinoKEY="1234";                         // must be the same with the server Virtuino KEY



#include <SoftwareSerial.h>
SoftwareSerial espSerial =  SoftwareSerial(2,3);      // arduino RX pin=2  arduino TX pin=3    connect the arduino RX pin to esp8266 module TX pin   -  connect the arduino TX pin to esp8266 module RX pin

//---------------------------------------------
unsigned long timeOfLastUpdate=0;

//==================================================== setup
//====================================================
void setup() {
  Serial.begin(9600); 
  while (!Serial) {;}
  espSerial.begin(9600);     // enable software serial
  while (!espSerial) {;}     // If your esp8266 module's speed is at 115200 you have to change it to 9600 before upoload this code  

  espSerial.println("AT+CWMODE=1");   // set esp8266 as client
  waitForResponse("OK",1000);

  espSerial.println("AT+CWJAP=\""+ssid+"\",\""+password+"\"");  // set your home router SSID and password
  waitForResponse("OK",5000);

  //espSerial.println("AT+CIPSTA=\"192.168.1.50\"");     // set a client IP only if the default client IP is the same with the server IP
  //waitForResponse("OK",5000);

  espSerial.println("AT+CIPSTA?");
  waitForResponse("OK",3000); 
  
  espSerial.println("AT+CIFSR");           // get ip address
  waitForResponse("OK",1000);
   
  Serial.println("\nSetup completed");
}



//==================================================== loop
//====================================================

void loop() {

   if (millis()-timeOfLastUpdate>5000) {    // update values to server every 5 seconds
    //----------- Add code here to read your sensor values
    int value1 = random(100);              // Replace with the value you want to send to virtuinoSE server

    //----------- Prepare the command data that the client has to send to VirtuinoSE server code   
    String commandBuffer="&KEY="+virtuinoKEY;   // Start a new command with the VirtuinoKey. If you don't want to use a key, start the command with an empty String
   
    commandBuffer+="&V0=?";                 // Command 1. Ask Virtuino server what is the value of memory V0
    commandBuffer+="&V1="+String(value1);   // Command 2. Send the value1 to memory V1
    commandBuffer+="&V2=?";                 // Command 2. Ask Virtuino server what is the value of memory V2
    
    String response=sendCommand(commandBuffer);  // send all the commands to virtuino server. Get the response
    Serial.println("Response="+response);
    
    String valueOf_V0_asString = getValue(0,&response);          // get the value of V0 as string from the response text
    if (valueOf_V0_asString.length()>0){                         
      float valueOf_V0= valueOf_V0_asString.toFloat();           // convert the text value to float or int
      Serial.println("valueOf_V0="+String(valueOf_V0));          // do something with value
    }

     String valueOf_V2_asString = getValue(2,&response);          // get the value of V2 as string 
    if (valueOf_V2_asString.length()>0){                         
      int valueOf_V2= valueOf_V2_asString.toInt();                // convert the text value to float or int
      Serial.println("valueOf_V2="+String(valueOf_V2));           // do something with value
    }

    timeOfLastUpdate=millis();
   }
   
}









//=============== Virtuino Client Library for ESP8266-01 (included) ===========
//==============================================================
//==============================================================
/* Only the next three functions are nedded to communicate with Virtuino server code
 * Avoid to make changes to the code below  
 */

//==================================================== sendCommand
//====================================================

String sendCommand(String commandData){
  String reply="";
  Serial.print("\nConnecting to ");
  Serial.println(virtuinoServerIP);

  String cmd = "AT+CIPSTART=\"TCP\",\"";       
  cmd += virtuinoServerIP;                          
  cmd += "\",";
  cmd +=virtuinoServerPort;
  espSerial.println(cmd);
  Serial.println(cmd);
  if(espSerial.find("Error")){
    Serial.println("AT+CIPSTART error");
    return "Error";
  }
  
  // prepare GET string
  String getStr = "GET /";   
  getStr += commandData;
  getStr += "\r\n\r\n HTTP/1.1\r\nHost: ";
  getStr +=  virtuinoServerIP;
  getStr +=  "\r\nConnection: close\r\n\r\n";
  
  // send data length
  cmd = "AT+CIPSEND=";
  cmd += String(getStr.length());
  espSerial.println(cmd);
  Serial.println(cmd);
  
  delay(100);
  if(espSerial.find(">")){
    espSerial.print(getStr);
    Serial.print(getStr);

    
    //--- get Response
     unsigned long timeout = millis();
    while (!espSerial.available()) {
    if (millis() - timeout > 5000) {
      Serial.println(">>> Client Timeout !");
      espSerial.println("AT+CIPCLOSE");
      return "Error";
    }
  }
  
    //Read the reply from server and print them to Serial
    int waitTime = 1000; //wait 1000 millis for the response
    long t=millis();
    char c;
    String response="";
    while (t+waitTime>millis()){
      if (espSerial.available()){
        c=espSerial.read();
        Serial.print(c);
        response+=c;
      }
    }
    
    // check if the response contains Virtuino command data
    int dataStartIndex= response.indexOf('&');   
    if (dataStartIndex!=-1) {
      int lastChar= response.indexOf('\r',dataStartIndex);     // clear Virtuino data form the response
      reply = response.substring(dataStartIndex,lastChar);
    }
   
    espSerial.println("AT+CIPCLOSE");
  }
  else{
    espSerial.println("AT+CIPCLOSE");
    Serial.println("Error");
    return "Error";
  }
  return reply;
}


//============================================================== getValue
String getValue(int memoryIndex, String* responseData){
   String s= "&V"+String(memoryIndex)+'=';
   int index = responseData->indexOf(s);
   if (index!=-1){
     int nextCommandIndex= responseData->indexOf('&',index+1);
     if (nextCommandIndex==-1) return responseData->substring(index+s.length());
     else return responseData->substring(index+s.length(),nextCommandIndex);
   }  
   return "";
}

//======================================================================================== waitForResponse
//========================================================================================
boolean waitForResponse(String target1,  int timeout){
    String data;
    char a;
    unsigned long start = millis();
    boolean rValue=false;
    while (millis() - start < timeout) {
        while(espSerial.available() > 0) {
            a = espSerial.read();
            Serial.print(a);
            if(a == '\0') continue;
            data += a;
        }
        if (data.indexOf(target1) != -1) {
            rValue=true;
            break;
        } 
    }
    return rValue;
}
  
//----------------------------------------------------------------------------
/* Virtuino GET request to upload values of V1 and V4 memories to board  (KEY=1234)
 *  GET /&KEY=1234&V1=12.3&V4=1
 *  Response: OK 
 *  
 * Virtuino GET request to read values from board (KEY=1234)
 *  GET /&KEY=1234&V2=?&V3=?
 *  Response: &V2=123&V3=1
 *  
 *  * Virtuino GET request to upload and read values from board (KEY=1234)
 *  GET /&KEY=1234&V2=45.6&V3=?
 *  Response: &V3=1
 *  
 *  Virtuino GET request to check the connection  (KEY=1234)
 *  GET /&KEY=1234&CNT=?
 *  Response: &CNT=Hello Virtuino I am the ESP8266 board
 *  
 * Virtuino GET request to upload value as TEXT from memory V1 to board  (KEY=1234)
 *  GET /&KEY=1234&T1=Hello
 *  Response: OK 

 * Virtuino GET request to read value a TEXT from board  (KEY=1234)
 *  GET /&KEY=1234&T1=?
 *  Response &T1=the text you want to send 
 *  or
 *  GET /&KEY=1234&V1=?
 *  Response &V1=the text you want to send 
 *  
 *  All the texts that are sent from Virtuino are encoded 
 *  All the texts that received from Virtuino will be decoded
 */

